#ifndef ZP_ASSEMBLY_GENERATOR_H_
#define ZP_ASSEMBLY_GENERATOR_H_

#include "Optimizer.h"

namespace zp
{
    class X64AssemblyHolder;

    class IRVisitor;

    class AssemblyGenerator;
}


class X64AssemblyHolder
{
public:
    enum class Register {
        RAX, RBX, RCX, RDX,
        RSI, RDI, RBP, RSP,
        R8, R9, R10, R11, R12, R13, R14, R15
    };

    void mov(Register src, Register target) = 0;
    void mov(uint64_t imm, Register target) = 0;
};


class zp::IRVisitor
{
public:
    virtual void visitBlockInstruction(const BlockInstruction&) = 0;
    virtual void visitUnionBlockInstruction(const UnionBlockInstruction&) = 0;
    virtual void visitIOInstruction(const IOInstruction&) = 0;
    virtual void visitSimpleBlockInstruction(const SimpleBlockInstruction&) = 0;
    virtual void visitLinearLoopInstruction(const LinearLoopInstruction&) = 0;
    virtual void visitLoopInstruction(const LoopInstruction&) = 0;
};


class zp::AssemblyGenerator :
    public IRVisitor
{
    X64AssemblyHolder& as;
public:

    inline AssemblyGenerator(X64AssemblyHolder& as) :
        as{ as } {}

    virtual void visitBlockInstruction(const BlockInstruction&);
    virtual void visitUnionBlockInstruction(const UnionBlockInstruction&);
    virtual void visitIOInstruction(const IOInstruction&);
    virtual void visitSimpleBlockInstruction(const SimpleBlockInstruction&);
    virtual void visitLinearLoopInstruction(const LinearLoopInstruction&);
    virtual void visitLoopInstruction(const LoopInstruction&);
};




#endif // ZP_ASSEMBLY_GENERATOR_H_
