#include "Optimizer.h"
#include "Parser.h"
#include "AssemblyGenerator.h"
#include <iostream>
#include <vector>

using namespace zp;
using namespace std;

unique_ptr<BlockInstruction> AstVisitor::visit(Block& block)
{
    return block.accept(*this);
}


void UnionBlockInstruction::accept(IRVisitor& v)
{
    v.visitUnionBlockInstruction(*this);
}


void IOInstruction::accept(IRVisitor& v)
{
    v.visitIOInstruction(*this);
}


void SimpleBlockInstruction::accept(IRVisitor& v)
{
    v.visitSimpleBlockInstruction(*this);
}


void LinearLoopInstruction::accept(IRVisitor& v)
{
    v.visitLinearLoopInstruction(*this);
}


void LoopInstruction::accept(IRVisitor& v)
{
    v.visitLoopInstruction(*this);
}


unique_ptr<BlockInstruction> Optimizer::visitInstructionBlock(InstructionBlock& ib)
{
    std::cout << "InstructionBlock" << std::endl;
    unique_ptr<SimpleBlockInstruction> sbi = make_unique<SimpleBlockInstruction>();
    for (auto instr : ib.instructions) {
        switch(instr) {
            case Instruction::LEFT:
                sbi->pointerMoved--;
                break;
            case Instruction::RIGHT:
                sbi->pointerMoved++;
                break;
            case Instruction::PLUS:
                sbi->added[sbi->pointerMoved]++;
                break;
            case Instruction::MINUS:
                sbi->added[sbi->pointerMoved]--;
                break;
            default:
                break;
        }
    }
    return sbi;
}


unique_ptr<BlockInstruction> Optimizer::visitIOBlock(IOBlock& io)
{
    return make_unique<IOInstruction>(io.ioInstruction == Instruction::COMMA);
}


unique_ptr<BlockInstruction> Optimizer::visitUnionBlock(UnionBlock& ub)
{
    auto ret = make_unique<UnionBlockInstruction>();
    ret->content.reserve(ub.instructions.size());
    for (auto& block : ub.instructions) {
        ret->content.push_back(visit(*block.get()));
    }
    return ret;
}


unique_ptr<BlockInstruction> Optimizer::visitLoop(Loop& loop)
{
    auto ret = make_unique<LoopInstruction>();
    ret->content.reserve(loop.instructions.size());
    bool noSubLoops = true;
    for (auto& block : loop.instructions) {
        auto v = visit(*block.get());
        if (!dynamic_cast<SimpleBlockInstruction*>(v.get())) {
            noSubLoops = false;
        }
        ret->content.push_back(std::move(v));
    }
    if (noSubLoops) {
        // TODO: apply linear loop optimizations
    }
    return ret;
}


